@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009 CNRS
+ * Copyright © 2009, 2024 CNRS
  * Copyright © 2009-2024 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
@@ -968,8 +968,9 @@ hwloc_utils_get_best_node_in_array_by_memattr(hwloc_topology_t topology, hwloc_m
 enum hwloc_utils_cpuset_format_e {
   HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN,
   HWLOC_UTILS_CPUSET_FORMAT_HWLOC,
-  HWLOC_UTILS_CPUSET_FORMAT_TASKSET,
-  HWLOC_UTILS_CPUSET_FORMAT_LIST
+  HWLOC_UTILS_CPUSET_FORMAT_LIST,
+  HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD,
+  HWLOC_UTILS_CPUSET_FORMAT_TASKSET
 };
 
 static __hwloc_inline enum hwloc_utils_cpuset_format_e
@@ -979,24 +980,100 @@ hwloc_utils_parse_cpuset_format(const char *string)
     return HWLOC_UTILS_CPUSET_FORMAT_HWLOC;
   else if (!strcmp(string, "list"))
     return HWLOC_UTILS_CPUSET_FORMAT_LIST;
+  else if (!strcmp(string, "systemd-dbus-api"))
+    return HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD;
   else if (!strcmp(string, "taskset"))
     return HWLOC_UTILS_CPUSET_FORMAT_TASKSET;
   else
     return HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN;
 }
 
+/* The AllowedCPUs and AllowedMemoryNodes systemd DBus API syntax expects a string as follows:
+ * "ay 0xNNNN 0xAA [0xBB [...]]"
+ * where:
+ *   "0xNNNN" is the size of the array, max size 2^26.
+ *   "0xAA [0xBB [...]]" is the cpuset or nodeset mask given bytes after bytes, little endian order.
+ * e.g. the output of `hwloc-calc --cpuset-input-format list 0,31-32,63-64,77 --cpuset-output-format systemd-dbus-api` is
+ *   "ay 0x000a 0x01 0x00 0x00 x80 0x01 0x00 0x00 0x80 0x01 0x20
+ * e.g. the output of `hwloc-calc node:0 node:1 --nodeset-output-format systemd-dbus-api` is
+ *   "ay 0x0001 0x03"
+ * */
+#define HWLOC_SYSTEMD_DBUS_API_PREFIX_FORMAT "ay 0x%04x"
+#define HWLOC_SYSTEMD_DBUS_API_PREFIX_SIZE 9
+#define HWLOC_SYSTEMD_DBUS_API_BYTES_FORMAT " 0x%02x"
+#define HWLOC_SYSTEMD_DBUS_API_BYTES_SIZE 5
+static __hwloc_inline int hwloc_utils_systemd_asprintf(char ** strp, const struct hwloc_bitmap_s * __hwloc_restrict set)
+{
+  int last = hwloc_bitmap_last(set);
+  if (last == -1 ) {
+    fprintf(stderr, "Empty and infinite sets are not supported with the systemd-dbus-api output format\n");
+    exit(EXIT_FAILURE);
+  }
+  int bytes = last / 8 + 1; /* how many bytes to represent the bit mask */
+  int buflen = HWLOC_SYSTEMD_DBUS_API_PREFIX_SIZE + HWLOC_SYSTEMD_DBUS_API_BYTES_SIZE * bytes + 1;
+  int ret = 0;
+  char *buf;
+  buf = malloc(buflen);
+  *strp = buf;
+  ret = snprintf(buf, buflen, HWLOC_SYSTEMD_DBUS_API_PREFIX_FORMAT, bytes);
+  unsigned long ith = 0;
+  for (int byte=0; byte < bytes; byte++) {
+    if (byte % HWLOC_SIZEOF_UNSIGNED_LONG == 0) {
+      ith = hwloc_bitmap_to_ith_ulong(set, byte / HWLOC_SIZEOF_UNSIGNED_LONG);
+    }
+    ret += snprintf(buf + ret, HWLOC_SYSTEMD_DBUS_API_BYTES_SIZE + 1, HWLOC_SYSTEMD_DBUS_API_BYTES_FORMAT, (unsigned char) (ith & 0xFF));
+    ith >>= 8;
+  }
+  assert(ith == 0); /* whenever some bytes of the ulong ith may not be consumed, they must be 0 */
+  return ret;
+}
+
 static __hwloc_inline int
 hwloc_utils_cpuset_format_asprintf(char **string, hwloc_const_bitmap_t set,
                                    enum hwloc_utils_cpuset_format_e cpuset_output_format)
 {
   switch (cpuset_output_format) {
   case HWLOC_UTILS_CPUSET_FORMAT_HWLOC: return hwloc_bitmap_asprintf(string, set);
-  case HWLOC_UTILS_CPUSET_FORMAT_TASKSET: return hwloc_bitmap_taskset_asprintf(string, set);
   case HWLOC_UTILS_CPUSET_FORMAT_LIST: return hwloc_bitmap_list_asprintf(string, set);
+  case HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD: return hwloc_utils_systemd_asprintf(string, set);
+  case HWLOC_UTILS_CPUSET_FORMAT_TASKSET: return hwloc_bitmap_taskset_asprintf(string, set);
   default: abort();
   }
 }
 
+static __hwloc_inline int
+hwloc_utils_cpuset_format_sscanf(hwloc_bitmap_t set, const char *string, enum hwloc_utils_cpuset_format_e cpuset_format)
+{
+  if (cpuset_format == HWLOC_UTILS_CPUSET_FORMAT_UNKNOWN) {
+    /* If user doesn't enforce a format, try to guess.
+     * There is some ambiguity if a list of singleton like 1,3,5 is given,
+     * it may be parsed as 0x1,0x3,0x5 (hwloc) or 1-1,3-3,5-5 (list).
+     * Assume list first, then hwloc, then taskset.
+     */
+    if (hwloc_strncasecmp(string, "0x", 2) && strchr(string, '-'))
+      cpuset_format = HWLOC_UTILS_CPUSET_FORMAT_LIST;
+    else if (strchr(string, ','))
+      cpuset_format = HWLOC_UTILS_CPUSET_FORMAT_HWLOC;
+    else
+      cpuset_format = HWLOC_UTILS_CPUSET_FORMAT_TASKSET;
+  }
+
+  switch (cpuset_format) {
+  case HWLOC_UTILS_CPUSET_FORMAT_HWLOC:
+    return hwloc_bitmap_sscanf(set, string);
+    break;
+  case HWLOC_UTILS_CPUSET_FORMAT_LIST:
+    return hwloc_bitmap_list_sscanf(set, string);
+    break;
+  case HWLOC_UTILS_CPUSET_FORMAT_TASKSET:
+    return hwloc_bitmap_taskset_sscanf(set, string);
+    break;
+  default:
+    /* HWLOC_UTILS_CPUSET_FORMAT_SYSTEMD input not supported */
+    abort();
+  }
+}
+
 static __hwloc_inline unsigned long
 hwloc_utils_parse_restrict_flags(char * str){
   struct hwloc_utils_parsing_flag possible_flags[] = {
